# Author : Guillaume Marques
# Description : Procedures to select subsets of satellites for which we separate SSI.

function satsubsetsfilter(nbsats::Int, λ_vals::Vector{Double})::Vector{Int}
  satsubsetsids = 1:(2^nbsats - 1)
  nbuts = Vector{Double}()
  satsubsetstoseparate = Vector{Int}()

  # Compute the number of urban trucks visiting each subset.
  for sss in satsubsetsids
    nbut = sum(λ_vals[routes_visiting(nbsats, sss)])
    push!(nbuts, nbut)
  end

  sss_vals = [(sss, count_ones(sss), nbuts[sss]) for sss in satsubsetsids]
  sss_vals_sorted = copy(sss_vals)

  sort!(sss_vals_sorted, by = val -> (val[3], val[2]), rev = true)

  nb_ut_used = sss_vals_sorted[1][3]
  last_kept_v_val = 0
  for (subsetid, subsetlen, v_val) in sss_vals_sorted
    keep_subset = true
    if subsetlen == nbsats || v_val == nb_ut_used     
      keep_subset = false
    elseif v_val < 1e-6
      keep_subset = false
    elseif v_val == last_kept_v_val 
      for wrap_set in wrappingsets(subsetid, nbsats)
        (wrap_subsetid, wrap_subsetlen, wrap_v_val) = sss_vals[wrap_set]
        if v_val + 0.1 >= wrap_v_val
          keep_subset = false
          break
        end
      end
    end
    if keep_subset
      last_kept_v_val = v_val
      push!(satsubsetstoseparate, subsetid)
    end
    # for debugging purpose
    #println(" > sss = \e[1m $subsetid \e[0m, length = \e[1m $subsetlen \e[0m,  v_val = \e[1m $v_val \e[0m. $(satsubsets[subsetid])")
    #keep_subset && println("\e[32m keep the subset \e[0m")
    #!keep_subset && println("\e[31m do not keep the subset \e[0m")
  end
  return random_selection(satsubsetstoseparate, 1000)
end

function random_selection(satsubsettab::Vector{Int}, esp_max_size::Int
    )::Vector{Int}
  len = length(satsubsettab)
  (len < esp_max_size) && return satsubsettab
  newsatsubsettab = Int[]
  for satsubset in satsubsettab
    p = rand(rng, 0:0.001:1)
    if p < esp_max_size / len
      push!(newsatsubsettab, satsubset)
    end
  end
  return newsatsubsettab
end
