# Author : Guillaume Marques
# Description : Datastructure to store violated inequalities and delete duplicates.

struct SatellitesSupplyInequality <: ViolatedInequality
  m_value::Double
  rhs_value::Double
  cov_vertices::BitArray
  uncov_vertices::BitArray
  satsubsetid::Int
  violation_value::Double
end

const SSIContainer = ViolatedInequalities{SatellitesSupplyInequality}

function duplicated(a::SatellitesSupplyInequality, 
  b::SatellitesSupplyInequality)::Bool
  return (a.violation_value == b.violation_value) && 
    (a.satsubsetid == b.satsubsetid) && 
    (a.cov_vertices == b.cov_vertices) &&
    (a.uncov_vertices == b.uncov_vertices) &&
    (a.rhs_value == b.rhs_value)
    (a.m_value == b.m_value)
end

function violationvalue(a::SatellitesSupplyInequality)::Double
  return a.violation_value
end

function getedges(ssi::SatellitesSupplyInequality)::Vector{Tuple{Int, Int}}
  edges = Vector{Tuple{Int, Int}}()
  n = length(ssi.cov_vertices)
  for i in 1:(n-1), j in i:n
    if (ssi.cov_vertices[i] && ssi.uncov_vertices[j]) || 
        (ssi.cov_vertices[j] && ssi.uncov_vertices[i])
      push!(edges, (i, j))
    end
  end
  return edges
end
