# Description
# A problem is the union of a formulation and a dataset
import JuMP.solve

mutable struct Problem{T <: GraphData}
  data::Data2EVRP{T}
  #formulation::Formulation
  model
  variables::Variables
  parameters::Dict{Symbol, Any}
  sep_metrics::Vector{SeparationMetrics}
end

getdata(p::Problem) = p.data
getinstance(p::Problem) = instancename(getdata(p))
getformulation(p::Problem) = p.formulation
getmodel(p::Problem) = p.model
getvariables(p::Problem) = p.variables
getsolver(p::Problem) = getsolver(getformulation(p))
getparameters(p::Problem) = p.parameters
getseparationmetrics(p::Problem) = p.sep_metrics

function restrictedmasterparams(instance::Data2EVRP)::String
  nb_customers = nbcustomers(instance)
  nb_satellites = nbsatellites(instance)
  coeff = 1
  (nb_customers == 300 && nb_satellites == 10) && (coeff = 5)
  (nb_satellites == 15) && (coeff = 10)
  maxtime = Int(ceil(nb_customers / 50) * 20 * coeff)
  polish = Int(ceil(maxtime / 2))
  return "--MaxTimeForRestrictedMasterIpHeur=$maxtime --PolishingAfterTimeInRestrictedMasterIpHeur=$polish"
end

function build_problem(bapcodconfigname::String, instancename::String, initub)::Problem
  appfolder = dirname(@__FILE__)
  configfile = "$appfolder/../../config/$bapcodconfigname"
  if !isfile(configfile)
    err = """
    Cannot find the file $configfile.
      > Path is TwoEchelonVehicleRouting/config/<arg>
    """
    error(err)
  end
  instance = read_instance(instancename)
  userparams = restrictedmasterparams(instance)
  #formulation = Formulation(instancename, configfile, userparams)
  print_messages_about_instance(instance)
  parameters = getparameters(instance, initub)
  model, variables, sep_ineq = build_model(instance, parameters)
  return Problem(instance, model, variables, parameters, sep_ineq)
end

function build_problem(formulationid::Integer, bapcodconfigname::String, instance::Data2EVRP)
  return build_problem(formulationid, bapcodconfigname, instance, :best)
end
