function reader_2ECVRP_set1(filename::String, instancename::String)
  scan = Scan(filename)
  lineid = Scanner.next(scan, String)
  nb_nodes = 0
  nb_sats = 0
  nb_custs = 0
  l1_cap = 0
  l2_cap = 0
  l1_fleet = 0
  l2_fleet = 0
  edges_weight_matrix = nothing
  nodes_demand = nothing
  cdc_id = 0
  while lineid != "EOF"
    if lineid == "DIMENSION"
      nb_nodes = Scanner.next(scan, Int)
    elseif lineid == "SATELLITES"
      nb_sats = Scanner.next(scan, Int)
    elseif lineid == "CUSTOMERS"
      nb_custs = Scanner.next(scan, Int)
    elseif lineid == "L1CAPACITY"
      l1_cap = Scanner.next(scan, Int)
    elseif lineid == "L2CAPACITY"
      l2_cap = Scanner.next(scan, Int)
    elseif lineid == "L1FLEET:"
      l1_fleet = Scanner.next(scan, Int)
    elseif lineid == "L2FLEET:"
      l2_fleet = Scanner.next(scan, Int)
    elseif lineid == "EDGE_WEIGHT_SECTION"
      edges_weight_matrix = nextmatrix(scan, Int, nb_nodes, nb_nodes)
    elseif lineid ∈ ["DEMAND_SECTION" "MAND_SECTION"]
      nodes_demand = nextmatrix(scan, Int, nb_nodes, 2)
    elseif lineid == "DEPOT_SECTION"
      cdc_id = Scanner.next(scan, Int)
    end
    lineid = Scanner.next(scan, String)
  end
  nodes = create_nodes_dataframe(nb_nodes, [cdc_id], [s for s in 1:nb_sats], [],
                                 nothing, view(nodes_demand, :, 2), nothing)
  vehicles = create_vehicles_dataframe(["A"], [l1_fleet], [l1_cap], nothing,
                                       ["B"], [l2_fleet], [l2_cap], nothing)
  return Data2ECVRP(nodes, vehicles, edges_weight_matrix, instancename)
end

function reader_2ECVRP_set2_and_set3(filename::String, instancename::String)
  scan = Scan(filename)
  lineid = Scanner.next(scan, String)
  nb_nodes = 0
  nb_sats = 0
  nb_custs = 0
  l1_cap = 0
  l2_cap = 0
  l1_fleet = 0
  l2_fleet = 0
  nodes_coords = nothing
  nodes_demand = nothing
  cdc_id = 0
  while lineid != "EOF"
    if lineid == "DIMENSION"
      nb_nodes = Scanner.next(scan, Int)
    elseif lineid == "SATELLITES"
      nb_sats = Scanner.next(scan, Int)
    elseif lineid == "CUSTOMERS"
      nb_custs = Scanner.next(scan, Int)
    elseif lineid == "L1CAPACITY"
      l1_cap = Scanner.next(scan, Int)
    elseif lineid == "L2CAPACITY"
      l2_cap = Scanner.next(scan, Int)
    elseif lineid == "L1FLEET:"
      l1_fleet = Scanner.next(scan, Int)
    elseif lineid == "L2FLEET:"
      l2_fleet = Scanner.next(scan, Int)
    elseif lineid == "NODE_COORD_SECTION"
      nodes_coords = nextmatrix(scan, Int, nb_nodes, 3)
    elseif lineid == "DEMAND_SECTION"
      nodes_demand = nextmatrix(scan, Int, nb_custs + 1, 2)
    elseif lineid == "DEPOT_SECTION"
      cdc_id = Scanner.next(scan, Int)
    end
    lineid = Scanner.next(scan, String)
  end
  sats_id = [s for s in (nb_nodes - nb_sats):nb_nodes]
  nodes = create_nodes_dataframe(nb_nodes, [cdc_id], sats_id, [],
                                 view(nodes_coords, :, 2:3), vcat(view(nodes_demand, :, 2), [0 for s in sats_id]), nothing)
  vehicles = create_vehicles_dataframe(["A"], [l1_fleet], [l1_cap], nothing,
                                       ["B"], [l2_fleet], [l2_cap], nothing)
  return Data2ECVRP(nodes, vehicles, instancename)
end

function reader_2ECVRP_set4(filename::String, instancename::String, satcap::Bool)
  scan = Scan(filename)
  lineid = Scanner.next(scan, String)
  nb_nodes = 0
  nb_sats = 0
  nb_custs = 0
  l1_cap = 0
  l2_cap = 0
  l1_fleet = 0
  l2_fleet = 0
  nodes = emptynodesframe4Data2EVRP()
  while lineid != "EOF"
    if lineid == "DIMENSION"
      nb_nodes = Scanner.next(scan, Int)
    elseif lineid == "SATELLITES"
      nb_sats = Scanner.next(scan, Int)
    elseif lineid == "CUSTOMERS"
      nb_custs = Scanner.next(scan, Int)
    elseif lineid == "L1CAPACITY"
      l1_cap = Scanner.next(scan, Int)
    elseif lineid == "L2CAPACITY"
      l2_cap = Scanner.next(scan, Int)
    elseif lineid == "L1FLEET:"
      l1_fleet = Scanner.next(scan, Int)
    elseif lineid == "L2FLEET:"
      l2_fleet = Scanner.next(scan, Int)
    elseif lineid == "c"
      n, x, y, d = nextarray(scan, Float32, 4)
      add_customer!(nodes, x, y, d)
    elseif lineid == "s"
      n, x, y, l2_veh_cap = nextarray(scan, Float32, 4)
      if !satcap
        l2_veh_cap = Inf
      end
      add_satellite!(nodes, x, y, 0, Inf, l2_veh_cap, 0, Inf)
    elseif lineid == "d"
      n, x, y, l1_veh_cap = nextarray(scan, Float32, 4)
      add_cdc!(nodes, x, y, 0, l1_veh_cap, 0, Inf)
    end
    lineid = Scanner.next(scan, String)
  end
  vehicles = create_vehicles_dataframe(["A"], [l1_fleet], [l1_cap], nothing,
                                       ["B"], [l2_fleet], [l2_cap], nothing)
  return Data2ECVRP(nodes, vehicles, instancename)
end

# http://www.univie.ac.at/prolog/research/TwoEVRP/
function reader_2ECVRP_univie_format(filename::String, instancename::String, nbsat, nbcustomers, handling_costs::Bool)
  scan = Scan(filename)
  ut_total, ut_cap, ut_costdist, ut_fixcost = nextarray(scan, Int, 4)
  cf_max_sat, cf_total, cf_cap, cf_costdist, cf_fixcost = nextarray(scan, Int, 5)
  vehicles = create_vehicles_dataframe(["A"], [ut_total], [ut_cap], nothing,
                                    ["B"], [cf_total], [cf_cap], nothing)
  nodes = emptynodesframe4Data2EVRP()
  cdc_x, cdc_y = nextarray(scan, Int, 2)
  h = 0.0
  if handling_costs
    h = Scanner.next(scan, Float64)
  end
  add_cdc!(nodes, cdc_x, cdc_y)
  for s in 1:nbsat
    sat_x, sat_y = nextarray(scan, Int, 2)
    h = 0
    if handling_costs
      h = Scanner.next(scan, Float64)
    end
    add_satellite!(nodes, sat_x, sat_y, h, Inf, cf_max_sat, 0, Inf)
  end
  for c in 1:nbcustomers
    cust_x, cust_y, d = nextarray(scan, Int, 3)
    add_customer!(nodes, cust_x, cust_y, d)
  end
  return Data2ECVRP(nodes, vehicles, instancename)
end

function reader_2ECVRP_set5(filename, instancename)
  # 2eVRP_<nbcustomers>-<nbsats>-<?>.dat
  m = match(r"2eVRP_(?P<nbc>\d+)-(?P<nbs>\d+)-\db?.dat$", instancename)
  nbs = parse(Int, m[:nbs])
  nbc = parse(Int, m[:nbc])
  return reader_2ECVRP_univie_format(filename, instancename, nbs, nbc, true)
end

function reader_2ECVRP_set6(filename, instancename)
  # A_n<nbcustomers>-<nbsats>.dat
  m = match(r"[A-C]-n(?P<nbc>\d+)-(?P<nbs>\d+).dat$", instancename)
  nbs = parse(Int, m[:nbs])
  nbc = parse(Int, m[:nbc]) - 1
  return reader_2ECVRP_univie_format(filename, instancename, nbs, nbc, true)
end

function reader_2ECVRP_set7(filename::String, instancename::String)
  m = match(r"2e-(?P<nbc>\d+)-(?P<nbs>\d+)-\d[a-z].dat$", instancename)
  nbs = parse(Int, m[:nbs])
  nbc = parse(Int, m[:nbc]) - 1
  return reader_2ECVRP_univie_format(filename, instancename, nbs, nbc, false)
end
