function create_nodes_dataframe(nb_nodes, cdcs, sats, depots, pos, demand, tw)
    nodes = emptynodesframe4Data2EVRP()
    for nodeid in 0:(nb_nodes - 1)
        # Type of node
        incdcs = findall(n -> n == nodeid, cdcs)
        insats = findall(n -> n == nodeid, sats)
        indepots = findall(n -> n == nodeid, depots)

        nodetype = "C"
        if length(incdcs) != 0
            nodetype = "D"
        elseif length(insats) != 0
            nodetype = "S"
        elseif length(indepots) != 0
            nodetype = "G"
        end
        # Position of the node
        x, y = NaN, NaN
        if pos != nothing
            x, y = pos[nodeid+1, 1], pos[nodeid+1, 2]
        end
        # Begin and end of the time window
        a, b = NaN, NaN
        if tw != nothing
            a, b =  tw[nodeid+1, 1], tw[nodeid+1, 2]
        end
        # Service time
        s = 0
        # Demand
        d = 0
        if demand != nothing
            d = demand[nodeid+1]
        end
        # handling cost at the node
        h = 0
        # capacity of vehicles
        cap1, cap2 = Inf, Inf
        push!(nodes, [nodetype, x, y, s, a, b, d, h, cap1, cap2, 0, Inf]) # Setup cost equals 0 & Items cap is Inf
    end
    return nodes
end

function add_cdc!(nodes, x, y, a, b, h, cap1, setup, itemcap)
    push!(nodes, ["D", x, y, 0, a, b, 0, h, cap1, 0, setup, itemcap])
end
add_cdc!(nodes, x, y) = add_cdc!(nodes, x, y, NaN, NaN, 0, Inf, 0, Inf)
add_cdc!(nodes, x, y, h, cap1, setup, itemcap) = add_cdc!(nodes, x, y, NaN, NaN, h, cap1, setup, itemcap)

function add_satellite!(nodes, x, y, st, a, b, h, cap1, cap2, setup, itemcap)
    push!(nodes, ["S", x, y, st, a, b, 0, h, cap1, cap2, setup, itemcap])
end
add_satellite!(nodes, x, y) = add_satellite!(nodes, x, y, 0, NaN, NaN, 0, Inf, Inf, 0, Inf)
add_satellite!(nodes, x, y, h) =  add_satellite!(nodes, x, y, 0, NaN, NaN, h, Inf, Inf, 0, Inf)
add_satellite!(nodes, x, y, h, cap1, cap2, setup, itemcap) = add_satellite!(nodes, x, y, 0, NaN, NaN, h, cap1, cap2, setup, itemcap)
add_satellite!(nodes, x, y, a, b, h, cap1, cap2, setup, itemcap) = add_satellite!(nodes, x, y, 0, a, b, h, cap1, cap2, setup, itemcap)

function add_customer!(nodes, x, y, st, a, b, d)
    push!(nodes, ["C", x, y, st, a, b, d, 0, Inf, Inf, 0, Inf])
end
add_customer!(nodes, x, y, d) = add_customer!(nodes, x, y, 0, NaN, NaN, d)

function set_vehicles_dataframe!(frame::DataFrame, level, types, fleets, caps, costs)
    for i in 1:length(types)
        cost = 0
        if costs != nothing
        cost = costs[i]
        end
        push!(frame, [types[i], level, cost, caps[i], fleets[i]])
    end
end

function create_vehicles_dataframe(l1_types, l1_fleets, l1_caps, l1_costs,
                                    l2_types, l2_fleets, l2_caps, l2_costs)
    vehicles = emptyvehiclesframe4Data2EVRP()
    set_vehicles_dataframe!(vehicles, 1, l1_types, l1_fleets, l1_caps, l1_costs)
    set_vehicles_dataframe!(vehicles, 2, l2_types, l2_fleets, l2_caps, l2_costs)
    return vehicles
end

include("2ECVRP.jl")
