import LightGraphs.add_edge!,
       LightGraphs.edges,
       LightGraphs.neighbors,
       LightGraphs.inneighbors,
       LightGraphs.outneighbors,
       LightGraphs.ne,
       LightGraphs.nv

# Graphs, arcs & edges
struct TierGraph{T}
  offset::Integer
  graph::T
end

# TODO move
const SimpleDiGraph = LightGraphs.SimpleGraphs.SimpleDiGraph{Int64}
const SimpleGraph = LightGraphs.SimpleGraphs.SimpleGraph{Int64}
# end TODO

lg(t::TierGraph) = t.graph
add_edge!(t::TierGraph, e) = add_edge!(t.graph, (e[1] - t.offset, e[2] - t.offset))
neighbors(t::TierGraph, v) = neighbors(t.graph, v - t.offset) .+ t.offset
incidents(t::TierGraph, v) = [sort(u, v) for u in neighbors(t, v)]
inneighbors(t::TierGraph{SimpleDiGraph}, v) = inneighbors(t.graph, v - t.offset) .+ t.offset
outneighbors(t::TierGraph{SimpleDiGraph}, v) = outneighbors(t.graph, v - t.offset) .+ t.offset
in_arcs(t::TierGraph{SimpleDiGraph}, v) = [(u, v) for u in inneighbors(t, v)]
out_arcs(t::TierGraph{SimpleDiGraph}, v) = [(v, u) for u in outneighbors(t, v)]
vertice(t::TierGraph) = (1 + t.offset):(nv(t.graph) + t.offset)
ne(t::TierGraph) = ne(t.graph)
nv(t::TierGraph) = nv(t.graph)
offset(t::TierGraph) = t.offset

function edges(t::TierGraph)
  set = Vector{Tuple}()
  for e in edges(t.graph)
    edge = Tuple(e)
    push!(set, (edge[1] + t.offset, edge[2] + t.offset))
  end
  return set
end

function edges(t::TierGraph, data, s::Int)
    set = Vector{Tuple}()
    C = customers(data)
    for i in C, j in C
        (i < j) && push!(set, (i, j))
    end
    for i in C
        push!(set, (i, s))
    end
    return set
end

sort(a, b) = (a < b) ? (a, b) : (b, a)

function complete_bipartite_edges!(graph::TierGraph{SimpleGraph}, set_a, set_b)
  for a in set_a, b in set_b
    add_edge!(graph, sort(a, b))
  end
end

function complete_bipartite_edges!(digraph::TierGraph{SimpleDiGraph}, set_src, set_dst)
  for a in set_src, b in set_dst
    add_edge!(digraph, (a, b))
  end
end

function complete_edges!(graph::TierGraph{SimpleGraph}, set)
  for i in set, j in set
    if i < j
      add_edge!(graph, (i, j))
    end
  end
end

function complete_edges!(graph::TierGraph{SimpleDiGraph}, set)
  for i in set, j in set
    if i != j
      add_edge!(graph, (i, j))
    end
  end
end
