using Scanner, DataFrames, CSV, DataStructures
using LightGraphs
using JuMP, VrpSolver
using CPLEX
using Statistics

import Base.iterate

include("app/misc.jl")
include("app/const.jl")
include("formulations/variants.jl")
include("app/load.jl")
include("data/types.jl")
include("graphs/graphs.jl")
include("data/2EVRP.jl")
include("graphs/2ECVRP.jl")
include("formulations/variables.jl")
include("app/parameters.jl")
include("validinequalities/metrics.jl")
include("validinequalities/container.jl")
include("app/builder.jl")
include("app/solutions.jl")
include("algorithms/algorithms.jl")
include("models/model.jl")
include("app/check_solution.jl")

#  ** Example **
#  julia src/run.jl TwoEchelonVehicleRouting/instances/Set2/E-n22-k4-s6-17.dat ub

function main(args)::Cvoid

    appfolder = dirname(@__FILE__)

    if (length(args) == 0)
        instancename = "Set3/E-n22-k4-s13-14.dat"
        initub = 530
    else
        instancename = args[1] #reading the first parameter passed through command line
        if (length(args) == 1)
            initub = 100000.0
        else
            initub = parse(Float64,args[2]) #reading the second parameter passed through command line
        end
    end

    problem = build_problem("2ECVRP.cfg", instancename, initub)
    configfile = "$appfolder/../config/2ECVRP.cfg"
    optimizer = VrpOptimizer(getmodel(problem), configfile)
    set_cutoff!(optimizer, initub)
    status = optimize!(optimizer)


    feasiblesolution = check_solution(optimizer, problem)
    !feasiblesolution && @warn("Final solution not feasible.")
    return
end

main(ARGS)
