# Author : Guillaume Marques
# Description : Data structure to store information about valid inequalities
# separation.

struct SeparationMetrics
  name::String
  separation_time::Vector{Double}
  nb_separated_ineq::Vector{Int}
  avg_violation::Vector{Double}
  max_violation::Vector{Double}
end

name(metrics::SeparationMetrics)::String = metrics.name
nb_sep_ineq(metrics::SeparationMetrics)::Int = sum(metrics.nb_separated_ineq)
sep_time(metrics::SeparationMetrics)::Double = sum(metrics.separation_time)

function avg_viol(metrics::SeparationMetrics)::Double
  (nb_sep_ineq(metrics) == 0) && return 0.0
  return mean(metrics.avg_violation)
end

function max_viol(metrics::SeparationMetrics)::Double
  (nb_sep_ineq(metrics) == 0) && return 0.0
  return maximum(metrics.max_violation)
end

function SeparationMetrics(name::String)
  return SeparationMetrics(name, Vector{Double}(), Vector{Int}(), 
      Vector{Double}(), Vector{Double}())
end

function register_metrics!(metrics::SeparationMetrics, time::Double, 
    nbsepineq::Int, avgviol::Double, maxviol::Double)::Cvoid
  push!(metrics.separation_time, time)
  push!(metrics.nb_separated_ineq, nbsepineq)
  push!(metrics.avg_violation, avgviol)
  push!(metrics.max_violation, maxviol)
  return
end
